<!DOCTYPE html>
<html>
<head>
    <meta charset="UTF-8">
    <title>Package Index</title>
</head>
<body>
{%- for package in packages | sort -%}
    {%- for filename in package.files | sort %}
    <a href="{{ filename }}">{{ filename }}</a><br>
    {%- endfor -%}
{%- endfor %}
</body>
</html>
